@@ -8,9 +8,10 @@
 #'
 #' @details
 #' The function `labels()` is a adaptation of the base function to the objects
-#' of class `declared`. In addition to the regular arguments, it has an
-#' additional logical one called `prefixed`, to retrieve the value labels
-#' prefixed with their values.
+#' of class `declared`. In addition to the regular arguments, it has two
+#' additional (logical) arguments called `prefixed` (FALSE by default), to 
+#' retrieve the value labels prefixed with their values, and `print_as_df`
+#' (TRUE by default) to print the result as a data frame.
 #'
 #' @return
 #' \code{labels()} will return a named vector.
@@ -181,28 +182,53 @@ label.data.frame <- function (x) {
 
 
 #' @export
-labels.declared <- function (object, prefixed = FALSE, ...) {
+labels.declared <- function (
+    object, prefixed = FALSE, print_as_df = TRUE, ...
+  ) {
     labels <- attr (object, "labels", exact = TRUE)
-    if (prefixed) {
-        names (labels) <- paste0 ("[", labels, "] ", names (labels))
+    
+    if (is.null(labels)) {
+      return(NULL)
     }
 
-    return (labels)
+    if (isTRUE(prefixed)) {
+      names (labels) <- paste0 ("[", labels, "] ", names (labels))
+    }
+
+    attr (labels, "print_as_df") <- isTRUE (print_as_df)
+      
+    return (
+      structure (labels, class = unique (c ("labels_df", class (labels))))
+    )
 }
 
 
 #' @export
-labels.haven_labelled_spss <- function (object, prefixed = FALSE, ...) {
+labels.haven_labelled_spss <- function (
+    object, prefixed = FALSE, print_as_df = TRUE, ...
+  ) {
+    
     labels <- attr (object, "labels", exact = TRUE)
-    if (prefixed)
-        names (labels) <- paste0 ("[", labels, "] ", names (labels))
-    labels
+    
+    if (is.null(labels)) {
+      return(NULL)
+    }
+
+    if (isTRUE(prefixed)) {
+      names (labels) <- paste0 ("[", labels, "] ", names (labels))
+    }
+
+    attr (labels, "print_as_df") <- isTRUE (print_as_df)
+      
+    return (
+      structure (labels, class = unique (c ("labels_df", class (labels))))
+    )
 }
 
 
 #' @export
-labels.data.frame <- function (object, prefixed = FALSE, ...) {
-    lapply (object, labels, prefixed = prefixed)
+labels.data.frame <- function (object, ...) {
+  lapply (object, labels, ...)
 }
 
 
